@@ -16,7 +16,7 @@ from ramalama.config import COLOR_OPTIONS, SUPPORTED_RUNTIMES
 try:
     import argcomplete
 
-    suppressCompleter = argcomplete.completers.SuppressCompleter
+    suppressCompleter: type[argcomplete.completers.SuppressCompleter] | None = argcomplete.completers.SuppressCompleter
 except Exception:
     suppressCompleter = None
 
@@ -26,9 +26,9 @@ import ramalama.oci
 from ramalama import engine
 from ramalama.chat import default_prefix
 from ramalama.common import accel_image, get_accel, perror
-from ramalama.config import CONFIG, coerce_to_bool
+from ramalama.config import CONFIG, coerce_to_bool, load_file_config
 from ramalama.logger import configure_logger, logger
-from ramalama.model import MODEL_TYPES, trim_model_name
+from ramalama.model import MODEL_TYPES, NoGGUFModelFileFound, SafetensorModelNotSupported, trim_model_name
 from ramalama.model_factory import ModelFactory, New
 from ramalama.model_inspect.error import ParseError
 from ramalama.model_store.global_store import GlobalModelStore
@@ -44,7 +44,6 @@ GENERATE_OPTIONS = ["quadlet", "kube", "quadlet/kube"]
 
 
 class ParsedGenerateInput:
-
     def __init__(self, gen_type: str, output_dir: str):
         self.gen_type = gen_type
         self.output_dir = output_dir
@@ -133,6 +132,10 @@ def get_parser():
 
 def init_cli():
     """Initialize the RamaLama CLI and parse command line arguments."""
+    # Need to know if we're running with --dryrun or --generate before adding the subcommands,
+    # otherwise calls to accel_image() when setting option defaults will cause unnecessary image pulls.
+    if any(arg in ("--dryrun", "--dry-run", "--generate") or arg.startswith("--generate=") for arg in sys.argv[1:]):
+        CONFIG.dryrun = True
     parser = get_parser()
     args = parse_arguments(parser)
     post_parse_setup(args)
@@ -276,6 +279,15 @@ def post_parse_setup(args):
         if not hasattr(args, "model"):
             args.model = args.MODEL
 
+    # Validate that --add-to-unit is only used with --generate and its format is <section>:<key>:<value>
+    if hasattr(args, "add_to_unit") and (add_to_units := args.add_to_unit):
+        if getattr(args, "generate", None) is None:
+            parser = get_parser()
+            parser.error("--add-to-unit can only be used with --generate")
+        if not (all(len([value for value in unit_to_add.split(":", 2) if value]) == 3 for unit_to_add in add_to_units)):
+            parser = get_parser()
+            parser.error("--add-to-unit parameters must be of the form <section>:<key>:<value>")
+
     if hasattr(args, "runtime_args"):
         args.runtime_args = shlex.split(args.runtime_args)
 
@@ -516,6 +528,7 @@ def _list_models(args):
 
 def info_cli(args):
     info = {
+        "Config": load_file_config(),
         "Engine": {
             "Name": args.engine,
         },
@@ -703,7 +716,7 @@ def _get_source_model(args):
     smodel = New(src, args)
     if smodel.type == "OCI":
         raise ValueError(f"converting from an OCI based image {src} is not supported")
-    if not smodel.exists():
+    if not smodel.exists() and not args.dryrun:
         smodel.pull(args)
     return smodel
 
@@ -780,6 +793,14 @@ def runtime_options(parser, command):
             choices=GENERATE_OPTIONS,
             help="generate specified configuration format for running the AI Model as a service",
         )
+        parser.add_argument(
+            "--add-to-unit",
+            dest="add_to_unit",
+            action='append',
+            type=str,
+            help="add KEY VALUE pair to generated unit file in the section SECTION (only valid with --generate)",
+            metavar="SECTION:KEY:VALUE",
+        )
         parser.add_argument(
             "--host",
             default=CONFIG.host,
@@ -823,6 +844,12 @@ If GPU device on host is accessible to via group access, this option leaks the u
         help="number of layers to offload to the gpu, if available",
         completer=suppressCompleter,
     )
+    parser.add_argument(
+        "--thinking",
+        default=CONFIG.thinking,
+        help="enable/disable thinking mode in reasoning models",
+        action=CoerceToBool,
+    )
     parser.add_argument(
         "--oci-runtime",
         help="override the default OCI runtime used to launch the container",
@@ -848,7 +875,7 @@ If GPU device on host is accessible to via group access, this option leaks the u
         choices=["always", "missing", "never", "newer"],
         help='pull image policy',
     )
-    if command in ["run", "serve"]:
+    if command in ["serve"]:
         parser.add_argument(
             "--rag", help="RAG vector database or OCI Image to be served with the model", completer=local_models
         )
@@ -917,6 +944,18 @@ def default_threads():
     return CONFIG.threads
 
 
+def chat_run_options(parser):
+    parser.add_argument(
+        '--color',
+        '--colour',
+        default="auto",
+        choices=get_args(COLOR_OPTIONS),
+        help='possible values are "never", "always" and "auto".',
+    )
+    parser.add_argument("--prefix", type=str, help="prefix for the user prompt", default=default_prefix())
+    parser.add_argument("--rag", type=str, help="a file or directory to use as context for the chat")
+
+
 def chat_parser(subparsers):
     parser = subparsers.add_parser("chat", help="OpenAI chat with the specified RESTAPI URL")
     parser.add_argument(
@@ -925,21 +964,13 @@ def chat_parser(subparsers):
         default=os.getenv("API_KEY"),
         help="OpenAI-compatible API key. Can also be set via the API_KEY environment variable.",
     )
-    parser.add_argument(
-        '--color',
-        '--colour',
-        default="auto",
-        choices=get_args(COLOR_OPTIONS),
-        help='possible values are "never", "always" and "auto".',
-    )
+    chat_run_options(parser)
     parser.add_argument(
         "--list",
         "--ls",
         action="store_true",
         help="list the available models at an endpoint",
     )
-    parser.add_argument("--prefix", type=str, help="prefix for the user prompt", default=default_prefix())
-    parser.add_argument("--rag", type=str, help="a file or directory to use as context for the chat")
     parser.add_argument("--url", type=str, default="http://127.0.0.1:8080/v1", help="the url to send requests to")
     parser.add_argument("--model", "-m", type=str, completer=local_models, help="model for inferencing")
     parser.add_argument(
@@ -951,14 +982,7 @@ def chat_parser(subparsers):
 def run_parser(subparsers):
     parser = subparsers.add_parser("run", help="run specified AI Model as a chatbot")
     runtime_options(parser, "run")
-    parser.add_argument(
-        '--color',
-        '--colour',
-        default="auto",
-        choices=get_args(COLOR_OPTIONS),
-        help='possible values are "never", "always" and "auto".',
-    )
-    parser.add_argument("--prefix", type=str, help="prefix for the user prompt", default=default_prefix())
+    chat_run_options(parser)
     parser.add_argument("MODEL", completer=local_models)  # positional argument
 
     parser.add_argument(
@@ -982,16 +1006,19 @@ def run_cli(args):
 
     try:
         model = New(args.MODEL, args)
+        model.ensure_model_exists(args)
         model.serve(args, quiet=True) if args.rag else model.run(args)
 
     except KeyError as e:
         logger.debug(e)
         try:
             args.quiet = True
-            model = ModelFactory(args.MODEL, args, ignore_stderr=True).create_oci()
-            model.serve(args) if args.rag else model.run(args)
-        except Exception:
-            raise e
+            oci_model = ModelFactory(args.MODEL, args, ignore_stderr=True).create_oci()
+            oci_model.ensure_model_exists(args)
+
+            oci_model.serve(args, quiet=True) if args.rag else oci_model.run(args)
+        except Exception as exc:
+            raise e from exc
 
 
 def serve_parser(subparsers):
@@ -1004,6 +1031,8 @@ def serve_parser(subparsers):
 def _get_rag(args):
     if os.path.exists(args.rag):
         return
+    if args.pull == "never" or args.dryrun:
+        return
     model = New(args.rag, args=args, transport="oci")
     if not model.exists():
         model.pull(args)
@@ -1081,7 +1110,7 @@ def rag_parser(subparsers):
         "--format",
         default=CONFIG.rag_format,
         help="Output format for RAG Data",
-        choices=["qdrant", "json", "markdown"],
+        choices=["qdrant", "json", "markdown", "milvus"],
     )
     parser.add_argument(
         "--image",
@@ -1120,7 +1149,9 @@ If GPU device on host is accessible to via group access, this option leaks the u
 Files/Directory containing PDF, DOCX, PPTX, XLSX, HTML, AsciiDoc & Markdown
 formatted files to be processed""",
     )
-    parser.add_argument("IMAGE", help="OCI Image name to contain processed rag data", completer=suppressCompleter)
+    parser.add_argument(
+        "DESTINATION", help="Path or OCI Image name to contain processed rag data", completer=suppressCompleter
+    )
     parser.add_argument(
         "--ocr",
         dest="ocr",
@@ -1132,7 +1163,7 @@ formatted files to be processed""",
 
 
 def rag_cli(args):
-    rag = ramalama.rag.Rag(args.IMAGE)
+    rag = ramalama.rag.Rag(args.DESTINATION)
     rag.generate(args)
 
 
@@ -1221,11 +1252,12 @@ def inspect_cli(args):
 
 
 def main():
-
     def eprint(e, exit_code):
         perror("Error: " + str(e).strip("'\""))
         sys.exit(exit_code)
 
+    parser: ArgumentParserWithDefaults
+    args: argparse.Namespace
     try:
         parser, args = init_cli()
         try:
@@ -1263,3 +1295,13 @@ def main():
         eprint(e, errno.EIO)
     except ParseError as e:
         eprint(f"Failed to parse model: {e}", errno.EINVAL)
+    except SafetensorModelNotSupported:
+        eprint(
+            f"""Safetensor models are not supported. Please convert it to GGUF via:
+$ ramalama convert --gguf=<quantization> {args.model} <oci-name>
+$ ramalama run <oci-name>
+""",
+            errno.EMEDIUMTYPE,
+        )
+    except NoGGUFModelFileFound:
+        eprint(f"No GGUF model file found for downloaded model '{args.model}'", errno.ENOMEDIUM)
