@@ -3,8 +3,10 @@ import platform
 import random
 import shlex
 import socket
+import subprocess
 import sys
 import time
+from abc import ABC, abstractmethod
 from typing import Optional
 
 import ramalama.chat as chat
@@ -22,7 +24,7 @@ from ramalama.common import (
 )
 from ramalama.config import CONFIG, DEFAULT_PORT, DEFAULT_PORT_RANGE
 from ramalama.console import should_colorize
-from ramalama.engine import Engine, dry_run
+from ramalama.engine import Engine, dry_run, wait_for_healthy
 from ramalama.kube import Kube
 from ramalama.logger import logger
 from ramalama.model_inspect.base_info import ModelInfoBase
@@ -53,15 +55,22 @@ with the default RamaLama
 $(error)s"""
 
 
-class NoRefFileFound(Exception):
+class NoGGUFModelFileFound(Exception):
+    pass
 
+
+class SafetensorModelNotSupported(Exception):
+    pass
+
+
+class NoRefFileFound(Exception):
     def __init__(self, model: str, *args):
         super().__init__(*args)
 
         self.model = model
 
     def __str__(self):
-        return f"No ref file or models found for '{self.model}'. Please pull model."
+        return f"No ref file found for '{self.model}'. Please pull model."
 
 
 def trim_model_name(model):
@@ -74,7 +83,10 @@ def trim_model_name(model):
     return model
 
 
-class ModelBase:
+class ModelBase(ABC):
+    model: str
+    type: str
+
     def __not_implemented_error(self, param):
         return NotImplementedError(f"ramalama {param} for '{type(self).__name__}' not implemented")
 
@@ -90,24 +102,31 @@ class ModelBase:
     def push(self, source_model, args):
         raise self.__not_implemented_error("push")
 
+    @abstractmethod
     def remove(self, args):
         raise self.__not_implemented_error("rm")
 
+    @abstractmethod
     def bench(self, args):
         raise self.__not_implemented_error("bench")
 
+    @abstractmethod
     def run(self, args):
         raise self.__not_implemented_error("run")
 
+    @abstractmethod
     def perplexity(self, args):
         raise self.__not_implemented_error("perplexity")
 
+    @abstractmethod
     def serve(self, args):
         raise self.__not_implemented_error("serve")
 
+    @abstractmethod
     def exists(self) -> bool:
         raise self.__not_implemented_error("exists")
 
+    @abstractmethod
     def inspect(self, args):
         raise self.__not_implemented_error("inspect")
 
@@ -115,15 +134,14 @@ class ModelBase:
 class Model(ModelBase):
     """Model super class"""
 
-    model = ""
-    type = "Model"
+    type: str = "Model"
 
-    def __init__(self, model, model_store_path):
+    def __init__(self, model: str, model_store_path: str):
         self.model = model
 
-        split = self.model.rsplit("/", 1)
-        self.directory = split[0] if len(split) > 1 else ""
-        self.filename = split[1] if len(split) > 1 else split[0]
+        split: list[str] = self.model.rsplit("/", 1)
+        self.directory: str = split[0] if len(split) > 1 else ""
+        self.filename: str = split[1] if len(split) > 1 else split[0]
 
         self._model_name: str
         self._model_tag: str
@@ -184,9 +202,19 @@ class Model(ModelBase):
         if dry_run:
             return "/path/to/model"
 
+        if self.model_type == 'oci':
+            if use_container or should_generate:
+                return os.path.join(MNT_DIR, 'model.file')
+            else:
+                return f"oci://{self.model}"
+
         ref_file = self.model_store.get_ref_file(self.model_tag)
-        if ref_file is None or not ref_file.model_files:
+        if ref_file is None:
             raise NoRefFileFound(self.model)
+        if not ref_file.model_files:
+            if any(file.name.endswith(".safetensors") for file in ref_file.files):
+                raise SafetensorModelNotSupported()
+            raise NoGGUFModelFileFound()
 
         # Use the first model file
         if is_split_file_model(self.model_name):
@@ -210,6 +238,9 @@ class Model(ModelBase):
         if dry_run:
             return ""
 
+        if self.model_type == 'oci':
+            return None
+
         ref_file = self.model_store.get_ref_file(self.model_tag)
         if ref_file is None:
             raise NoRefFileFound(self.model)
@@ -231,6 +262,9 @@ class Model(ModelBase):
         if dry_run:
             return ""
 
+        if self.model_type == 'oci':
+            return None
+
         ref_file = self.model_store.get_ref_file(self.model_tag)
         if ref_file is None:
             raise NoRefFileFound(self.model)
@@ -258,8 +292,11 @@ class Model(ModelBase):
     def base(self, args, name):
         # force accel_image to use -rag version. Drop TAG if it exists
         # so that accel_image will add -rag to the image specification.
-        if args.image == self.default_image and getattr(args, "rag", None):
+        if getattr(args, "rag", None) and not getattr(args, "image_override", False):
             args.image = rag_image(args.image)
+        if self.type == "Ollama":
+            args.UNRESOLVED_MODEL = args.MODEL
+            args.MODEL = self.resolve_model()
         self.engine = Engine(args)
         if args.subcommand == "run" and not getattr(args, "ARGS", None) and sys.stdin.isatty():
             self.engine.add(["-i"])
@@ -319,6 +356,12 @@ class Model(ModelBase):
         if args.dryrun:
             return
 
+        if self.model_type == 'oci':
+            if not self.engine.use_podman:
+                raise NotImplementedError("Serving OCI models via image mount is only supported with Podman.")
+            self.engine.add([f"--mount=type=image,src={self.model},destination={MNT_DIR},subpath=/models,rw=false"])
+            return None
+
         ref_file = self.model_store.get_ref_file(self.model_tag)
         if ref_file is None:
             raise NoRefFileFound(self.model)
@@ -338,13 +381,11 @@ class Model(ModelBase):
     def bench(self, args):
         self.ensure_model_exists(args)
         exec_args = self.build_exec_args_bench(args)
-        self.validate_args(args)
         self.execute_command(exec_args, args)
 
     def run(self, args):
         # The Run command will first launch a daemonized service
         # and run chat to communicate with it.
-        self.validate_args(args)
 
         args.port = compute_serving_port(args, quiet=args.debug)
         if args.container:
@@ -352,6 +393,7 @@ class Model(ModelBase):
 
         args.noout = not args.debug
 
+        self.ensure_model_exists(args)
         pid = os.fork()
         if pid == 0:
             # Child process - start the server
@@ -391,6 +433,14 @@ class Model(ModelBase):
         if status != 0:
             raise ValueError(f"Failed to serve model {self.model_name}, for ramalama run command")
 
+        if not args.dryrun:
+            try:
+                wait_for_healthy(args)
+            except subprocess.TimeoutExpired as e:
+                logger.error(f"Failed to serve model {self.model_name}, for ramalama run command")
+                logger.error(f"{e}: logs: {e.output}")
+                raise
+
         args.ignore = getattr(args, "dryrun", False)
         for i in range(6):
             try:
@@ -416,7 +466,7 @@ class Model(ModelBase):
                     chat.chat(args)
                     break
                 else:
-                    logger.debug(f"MLX server not ready, waiting... (attempt {i+1}/{max_retries})")
+                    logger.debug(f"MLX server not ready, waiting... (attempt {i + 1}/{max_retries})")
                     time.sleep(3)
                     continue
 
@@ -425,7 +475,7 @@ class Model(ModelBase):
                     perror(f"Error: Failed to connect to MLX server after {max_retries} attempts: {e}")
                     self._cleanup_server_process(args.pid2kill)
                     raise e
-                logger.debug(f"Connection attempt failed, retrying... (attempt {i+1}/{max_retries}): {e}")
+                logger.debug(f"Connection attempt failed, retrying... (attempt {i + 1}/{max_retries}): {e}")
                 time.sleep(3)
 
         args.initial_connection = False
@@ -489,7 +539,6 @@ class Model(ModelBase):
         return exec_args
 
     def perplexity(self, args):
-        self.validate_args(args)
         self.ensure_model_exists(args)
         exec_args = self.build_exec_args_perplexity(args)
         self.execute_command(exec_args, args)
@@ -514,6 +563,8 @@ class Model(ModelBase):
         return all
 
     def ensure_model_exists(self, args):
+        self.validate_args(args)
+
         if args.dryrun or self.exists():
             return
 
@@ -547,7 +598,7 @@ class Model(ModelBase):
         # If --nocontainer=False was specified return valid
         if args.container:
             return
-        if args.privileged:
+        if getattr(args, "privileged", None):
             raise KeyError(
                 "--nocontainer and --privileged options conflict. The --privileged option requires a container."
             )
@@ -587,16 +638,17 @@ class Model(ModelBase):
             self._get_entry_model_path(args.container, args.generate, args.dryrun),
             "--no-warmup",
         ]
+        if not args.thinking:
+            exec_args += ["--reasoning-budget", "0"]
         mmproj_path = self._get_mmproj_path(args.container, args.generate, args.dryrun)
         if mmproj_path is not None:
             exec_args += ["--mmproj", mmproj_path]
         else:
             exec_args += ["--jinja"]
 
-            # TODO: see https://github.com/containers/ramalama/issues/1202
-            # chat_template_path = self._get_chat_template_path(args.container, args.generate, args.dryrun)
-            # if chat_template_path is not None:
-            #     exec_args += ["--chat-template-file", chat_template_path]
+            chat_template_path = self._get_chat_template_path(args.container, args.generate, args.dryrun)
+            if chat_template_path is not None:
+                exec_args += ["--chat-template-file", chat_template_path]
 
         if should_colorize():
             exec_args += ["--log-colors"]
@@ -683,17 +735,19 @@ class Model(ModelBase):
         return exec_args
 
     def generate_container_config(self, args, exec_args):
-
         # Get the blob paths (src) and mounted paths (dest)
         model_src_path = self._get_entry_model_path(False, False, args.dryrun)
         chat_template_src_path = self._get_chat_template_path(False, False, args.dryrun)
+        mmproj_src_path = self._get_mmproj_path(False, False, args.dryrun)
         model_dest_path = self._get_entry_model_path(True, True, args.dryrun)
         chat_template_dest_path = self._get_chat_template_path(True, True, args.dryrun)
+        mmproj_dest_path = self._get_mmproj_path(True, True, args.dryrun)
 
         if args.generate.gen_type == "quadlet":
             self.quadlet(
                 (model_src_path, model_dest_path),
                 (chat_template_src_path, chat_template_dest_path),
+                (mmproj_src_path, mmproj_dest_path),
                 args,
                 exec_args,
                 args.generate.output_dir,
@@ -702,6 +756,7 @@ class Model(ModelBase):
             self.kube(
                 (model_src_path, model_dest_path),
                 (chat_template_src_path, chat_template_dest_path),
+                (mmproj_src_path, mmproj_dest_path),
                 args,
                 exec_args,
                 args.generate.output_dir,
@@ -710,6 +765,7 @@ class Model(ModelBase):
             self.quadlet_kube(
                 (model_src_path, model_dest_path),
                 (chat_template_src_path, chat_template_dest_path),
+                (mmproj_src_path, mmproj_dest_path),
                 args,
                 exec_args,
                 args.generate.output_dir,
@@ -731,7 +787,6 @@ class Model(ModelBase):
             raise NotImplementedError(file_not_found % {"cmd": exec_args[0], "error": str(e).strip("'")})
 
     def serve(self, args, quiet=False):
-        self.validate_args(args)
         self.ensure_model_exists(args)
 
         args.port = compute_serving_port(args, quiet=quiet or args.generate)
@@ -753,23 +808,23 @@ class Model(ModelBase):
 
         self.execute_command(exec_args, args)
 
-    def quadlet(self, model_paths, chat_template_paths, args, exec_args, output_dir):
-        quadlet = Quadlet(self.model_name, model_paths, chat_template_paths, args, exec_args)
+    def quadlet(self, model_paths, chat_template_paths, mmproj_paths, args, exec_args, output_dir):
+        quadlet = Quadlet(self.model_name, model_paths, chat_template_paths, mmproj_paths, args, exec_args)
         for generated_file in quadlet.generate():
             generated_file.write(output_dir)
 
-    def quadlet_kube(self, model_paths, chat_template_paths, args, exec_args, output_dir):
-        kube = Kube(self.model_name, model_paths, chat_template_paths, args, exec_args)
+    def quadlet_kube(self, model_paths, chat_template_paths, mmproj_paths, args, exec_args, output_dir):
+        kube = Kube(self.model_name, model_paths, chat_template_paths, mmproj_paths, args, exec_args)
         kube.generate().write(output_dir)
 
-        quadlet = Quadlet(self.model_name, model_paths, chat_template_paths, args, exec_args)
+        quadlet = Quadlet(kube.name, model_paths, chat_template_paths, mmproj_paths, args, exec_args)
         quadlet.kube().write(output_dir)
 
-    def kube(self, model_paths, chat_template_paths, args, exec_args, output_dir):
-        kube = Kube(self.model_name, model_paths, chat_template_paths, args, exec_args)
+    def kube(self, model_paths, chat_template_paths, mmproj_paths, args, exec_args, output_dir):
+        kube = Kube(self.model_name, model_paths, chat_template_paths, mmproj_paths, args, exec_args)
         kube.generate().write(output_dir)
 
-    def inspect(self, args):
+    def inspect(self, args) -> None:
         self.ensure_model_exists(args)
 
         model_name = self.filename
