@@ -4,16 +4,16 @@ import urllib.error
 from collections import Counter
 from http import HTTPStatus
 from pathlib import Path
-from typing import Optional, Tuple
+from typing import Optional, Sequence, Tuple
 
 import ramalama.model_store.go2jinja as go2jinja
-from ramalama.common import generate_sha256, perror, verify_checksum
+from ramalama.common import perror, sanitize_filename, verify_checksum
 from ramalama.endian import EndianMismatchError, get_system_endianness
 from ramalama.logger import logger
 from ramalama.model_inspect.gguf_parser import GGUFInfoParser, GGUFModelInfo
 from ramalama.model_store.constants import DIRECTORY_NAME_BLOBS, DIRECTORY_NAME_REFS, DIRECTORY_NAME_SNAPSHOTS
 from ramalama.model_store.global_store import GlobalModelStore
-from ramalama.model_store.reffile import RefFile, RefJSONFile, StoreFile, StoreFileType
+from ramalama.model_store.reffile import RefJSONFile, StoreFile, StoreFileType, migrate_reffile_to_refjsonfile
 from ramalama.model_store.snapshot_file import (
     LocalSnapshotFile,
     SnapshotFile,
@@ -22,10 +22,6 @@ from ramalama.model_store.snapshot_file import (
 )
 
 
-def sanitize_filename(filename: str) -> str:
-    return filename.replace(":", "-")
-
-
 def map_to_store_file_type(snapshot_type: SnapshotFileType) -> StoreFileType:
     ftype = StoreFileType.OTHER
     if snapshot_type == SnapshotFileType.Model:
@@ -38,35 +34,6 @@ def map_to_store_file_type(snapshot_type: SnapshotFileType) -> StoreFileType:
     return ftype
 
 
-def map_ref_file(ref_file: RefFile, snapshot_directory: str) -> RefJSONFile:
-    ref = RefJSONFile(
-        hash=ref_file.hash,
-        path=f"{ref_file.path}.json",
-        files=[],
-    )
-
-    def determine_type(filename: str) -> StoreFileType:
-        if filename == ref_file.model_name:
-            return StoreFileType.GGUF_MODEL
-        if filename == ref_file.chat_template_name:
-            return StoreFileType.CHAT_TEMPLATE
-        if filename == ref_file.mmproj_name:
-            return StoreFileType.MMPROJ
-        return StoreFileType.OTHER
-
-    def determine_blob_hash(filename: str) -> str:
-        blob_path = Path(os.path.join(snapshot_directory, sanitize_filename(ref_file.hash), filename)).resolve()
-        if not os.path.exists(blob_path):
-            return generate_sha256(filename)
-        return blob_path.stem
-
-    for file in ref_file.filenames:
-        ftype = determine_type(file)
-        ref.files.append(StoreFile(determine_blob_hash(file), file, ftype))
-
-    return ref
-
-
 class ModelStore:
     def __init__(
         self,
@@ -119,28 +86,23 @@ class ModelStore:
         return os.path.join(self.refs_directory, f"{model_tag}.json")
 
     def get_ref_file(self, model_tag: str) -> Optional[RefJSONFile]:
-        # Check if a ref file in old format is present by removing the file extension
-        old_ref_file_path = self.get_ref_file_path(model_tag).replace(".json", "")
-        if os.path.exists(old_ref_file_path):
-            ref = map_ref_file(RefFile.from_path(old_ref_file_path), self.snapshots_directory)
-            ref.write_to_file()
-            try:
-                os.remove(old_ref_file_path)
-            except Exception as ex:
-                logger.debug(f"Failed to remove old ref file '{old_ref_file_path}'\n: {ex}")
-
+        ref_file_path = self.get_ref_file_path(model_tag)
+        ref = migrate_reffile_to_refjsonfile(ref_file_path, self.snapshots_directory)
+        if ref is not None:
             return ref
 
-        ref_file_path = self.get_ref_file_path(model_tag)
         if not os.path.exists(ref_file_path):
             return None
 
         return RefJSONFile.from_path(ref_file_path)
 
     def update_ref_file(
-        self, model_tag: str, snapshot_hash: str = "", snapshot_files: list[SnapshotFile] = []
+        self, model_tag: str, snapshot_hash: str = "", snapshot_files: Optional[list[SnapshotFile]] = None
     ) -> Optional[RefJSONFile]:
-        ref_file: RefJSONFile = self.get_ref_file(model_tag)
+        if snapshot_files is None:
+            snapshot_files = []
+
+        ref_file: RefJSONFile | None = self.get_ref_file(model_tag)
         if ref_file is None:
             return None
 
@@ -195,9 +157,9 @@ class ModelStore:
         )
 
     def get_cached_files(self, model_tag: str) -> Tuple[str, list[str], bool]:
-        cached_files = []
+        cached_files: list[str] = []
 
-        ref_file: RefJSONFile = self.get_ref_file(model_tag)
+        ref_file: RefJSONFile | None = self.get_ref_file(model_tag)
         if ref_file is None:
             return ("", cached_files, False)
 
@@ -223,8 +185,10 @@ class ModelStore:
         snapshot_directory = self.get_snapshot_directory(snapshot_hash)
         os.makedirs(snapshot_directory, exist_ok=True)
 
-    def _download_snapshot_files(self, model_tag: str, snapshot_hash: str, snapshot_files: list[SnapshotFile]):
-        ref_file = self.get_ref_file(model_tag)
+    def _download_snapshot_files(self, model_tag: str, snapshot_hash: str, snapshot_files: Sequence[SnapshotFile]):
+        ref_file: None | RefJSONFile = self.get_ref_file(model_tag)
+        if ref_file is None:
+            raise ValueError("Cannot download snapshots without a valid ref file.")
 
         for file in snapshot_files:
             dest_path = self.get_blob_file_path(file.hash)
@@ -264,8 +228,8 @@ class ModelStore:
             if file.type == SnapshotFileType.ChatTemplate:
                 chat_template_file_path = self.get_blob_file_path(file.hash)
                 chat_template = ""
-                with open(chat_template_file_path, "r") as file:
-                    chat_template = file.read()
+                with open(chat_template_file_path, "r") as template_file:
+                    chat_template = template_file.read()
 
                 if not go2jinja.is_go_template(chat_template):
                     return
@@ -362,7 +326,7 @@ class ModelStore:
             self.remove_snapshot(model_tag)
             raise ex
 
-    def update_snapshot(self, model_tag: str, snapshot_hash: str, new_snapshot_files: list[SnapshotFile]) -> bool:
+    def update_snapshot(self, model_tag: str, snapshot_hash: str, new_snapshot_files: Sequence[SnapshotFile]) -> bool:
         validate_snapshot_files(new_snapshot_files)
         snapshot_hash = sanitize_filename(snapshot_hash)
 
@@ -404,7 +368,7 @@ class ModelStore:
             for entry in os.listdir(self.refs_directory)
             if os.path.isfile(os.path.join(self.refs_directory, entry))
         ]
-        refs = [self.get_ref_file(tag) for tag in model_tags]
+        refs = [ref for tag in model_tags if (ref := self.get_ref_file(tag))]
 
         blob_refcounts = Counter(file.name for ref in refs for file in ref.files)
 
